"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClassType = void 0;
const typescript_1 = require("typescript");
class ClassType {
    constructor(node, typeChecker) {
        if (node.kind !== typescript_1.SyntaxKind.ClassDeclaration) {
            throw new Error('Node is not a class declaration');
        }
        this.inheritedClasses = this.extractInheritedClasses(node, typeChecker);
    }
    extractInheritedClasses(node, typeChecker) {
        const baseTypes = typeChecker.getTypeAtLocation(node).getBaseTypes() || [];
        return baseTypes.reduce((classDeclarations, baseType) => {
            const symbol = baseType.getSymbol();
            if (!symbol) {
                return classDeclarations;
            }
            const declarations = symbol.getDeclarations();
            if (!declarations) {
                return classDeclarations;
            }
            for (const declaration of declarations) {
                if (this.isClassDeclaration(declaration)) {
                    classDeclarations.push(declaration);
                }
            }
            return classDeclarations;
        }, []);
    }
    isClassDeclaration(node) {
        return node.kind === typescript_1.SyntaxKind.ClassDeclaration;
    }
    hasInheritance() {
        return this.inheritedClasses.length > 0;
    }
    inheritsOnlyAbstractClasses() {
        return this.inheritedClasses.every((classDeclaration) => {
            var _a;
            return (_a = classDeclaration.modifiers) === null || _a === void 0 ? void 0 : _a.some((modifier) => modifier.kind === typescript_1.SyntaxKind.AbstractKeyword);
        });
    }
}
exports.ClassType = ClassType;
//# sourceMappingURL=class-type.js.map