"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.rule = exports.name = void 0;
const utils_1 = require("@typescript-eslint/utils");
const class_type_1 = require("./class-type");
const createRule = utils_1.ESLintUtils.RuleCreator(() => `https://github.com/lsndr/eslint-plugin-typescript-inheritance`);
exports.name = 'no-class-inheritance';
const defaultOptions = [
    {
        noInheritanceOfAbstractClasses: false,
    },
];
const meta = {
    docs: {
        description: 'Inheritance is not allowed.',
    },
    messages: {
        inheritanceNotAllowed: 'Inheritance of non-abstract classes is not allowed. Consider using composition or extend abstract class',
        inheritanceOfAbstractClassesNotAllowed: 'Inheritance is not allowed',
    },
    hasSuggestions: true,
    type: 'suggestion',
    schema: [
        {
            type: 'object',
            properties: {
                noInheritanceOfAbstractClasses: {
                    type: 'boolean',
                },
            },
            additionalProperties: false,
        },
    ],
};
exports.rule = createRule({
    meta,
    name: exports.name,
    defaultOptions,
    create(context, options) {
        return {
            ClassDeclaration(node) {
                var _a;
                const services = utils_1.ESLintUtils.getParserServices(context);
                const tsNode = services.esTreeNodeToTSNodeMap.get(node);
                const checker = services.program.getTypeChecker();
                const classType = new class_type_1.ClassType(tsNode, checker);
                if (!classType.hasInheritance()) {
                    return;
                }
                if ((_a = options[0]) === null || _a === void 0 ? void 0 : _a.noInheritanceOfAbstractClasses) {
                    context.report({
                        messageId: 'inheritanceOfAbstractClassesNotAllowed',
                        loc: node.loc,
                    });
                }
                else if (!classType.inheritsOnlyAbstractClasses()) {
                    context.report({
                        messageId: 'inheritanceNotAllowed',
                        loc: node.loc,
                    });
                }
            },
        };
    },
});
//# sourceMappingURL=no-class-inheritance.js.map